// Project 23 - Falling-edge external interrupt
// blink PORTB. If button is pressed, turn on PORTB for 2 seconds

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

ISR (INT0_vect) 
{   // Code to be executed when interrupt triggered
    PORTB = 0b11111111;    
    _delay_ms(2000);
    PORTB = 0b00000000;   
    EIFR = 0b00000001;
}

void startInt0()
{
    // initialize interrupt 0 (PD2/INT0/pin 4)
    // falling-edge (HIGH to LOW at pin 4)
    EICRA = 0b00000010;

    // turn on interrupt INT0
    EIMSK = 0b00000001;
    // enable global interrupt enable flag for interrupts to be processed
    sei();
}

int main(void)
{
    // declare global variables
    // setup GPIO pins etc.
    DDRB = 0b11111111; // set PORTB register as outputs
    DDRD = 0b00000000; // set PORTD pins 4 and 5 as inputs

    // initialize interrupt 
    startInt0();

    for(;;)
    {
        PORTB = 0b00000001;    
        _delay_ms(50);
        PORTB = 0b00000000;    
        _delay_ms(50);
    }
    return 0;              
}
