// Project 26 – Up/Down Counter with Interrupts

#include <avr/io.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define USART_BAUDRATE 4800
#define UBRR_VALUE 12

volatile uint8_t i = 100;       // initial value for counter
void USARTInit(void)
{
    // Set baud rate registers
    UBRR0H = (uint8_t)(UBRR_VALUE>>8);
    UBRR0L = (uint8_t)UBRR_VALUE;

    // Set data frame format to 8 data bits, no parity, 1 stop bit
    UCSR0C |= (1<<UCSZ01)|(1<<UCSZ00);

    // enable transmission and reception
    UCSR0B |= (1<<RXEN0)|(1<<TXEN0);
}

void USARTSendByte(uint8_t u8Data)
{
    // wait while previous byte is has been sent
    while(!(UCSR0A&(1<<UDRE0))){};
    // Transmit data
    UDR0 = u8Data;
}

void sendString(char myString[])  
{
    uint8_t a = 0;
    while (myString[a])
    {
        USARTSendByte(myString[a]);
        a++;
    }
}

ISR (INT0_vect)
{   // Code to be executed when interrupt INT0 triggered
     i = i - 1; // subtract one from the counter
     EIFR &= ~(1<<0);   // Set interrupt flag register for INT0 to zero
}

ISR (INT1_vect)
{   // Code to be executed when interrupt INT1 triggered
     i = i + 1; // add one to the counter
     EIFR &= ~(1<<1);   // Set interrupt flag register for INT1 to zero
}

void startInts()
{
    // initialize interrupt 0 (PD2/INT0/pin 4)
    // rising-edge (LOW to HIGH at pin 4)
    // initialize interrupt 1 (PD3/INT1/pin 5)
    // rising-edge (LOW to HIGH at pin 5)
    EICRA = 0b00001111;

    // turn on interrupt INT0 and INT1
    EIMSK = 0b00000011;
    // enable global interrupt enable flag
    sei();
}

int main(void)
{
    char a[10] = "Count - ";  // make sure you have " instead of “
    char s[10] = ""; // for our itoa conversion
    char newline[] = "\r\n";

    // setup pins 4 and 5 as inputs for INT0 and INT1.
    DDRD = 0b00000000;

    // initialize interrupts
    startInts();

    // initialize USART
    USARTInit();

    for(;;)
    {
        // send the value of our counter to the USART for display on the PC
        itoa(i, s, 10);  
        sendString(a);
        sendString(s);
        sendString(newline);
        _delay_ms(1000);
   }
   return 0;              
}
