// Project 35 – Temperature-controlled Fan

#include <avr/io.h>
#include <util/delay.h>

void startADC()
{
    ADMUX |= (1 << REFS0);                 // Use AVcc pin with ADC
    ADMUX |= (1 << MUX2) | (1 << MUX0);    // use ADC5 (pin 28)
    ADCSRA |= (1 << ADPS1) | (1 << ADPS0); // prescaler for 1MHz (/8)
    ADCSRA |= (1 << ADEN);                 // enable ADC
}

void initPWM(void) 
{
    // Timer 1A and 1B
    TCCR1A |= (1 << WGM10);  // Fast PWM mode
    TCCR1B |= (1 << WGM12);  // Fast PWM mode
    TCCR1B |= (1 << CS11);   
}

void motorOff(void)
{
    TCCR1A &= ~(1 << COM1A1);  // disconnect PWM from PB1
    PORTB &= ~(1 << PORTB1);   // set PB1 off
}

void motorOn(void)
{
    TCCR1A &= ~(1 << COM1A1);  // disconnect PWM from PB1
    PORTB |= (1 << PORTB1);    // set PB1 on
}

void motorPWM(uint8_t duty)
{
    TCCR1A |= (1 << COM1A1);   // connect PWM to OCR1A - PB1 
    OCR1A = duty;
}

void setup()
{
}

int main(void)
{
    DDRB |= (1 << PORTB1); // set PORTB pin 1 as output
    initPWM();
    startADC();
   
    uint8_t ADCvalue;
    float voltage;
    float temperature;   
   
    _delay_ms(3000); // delay motor action for a few moments on start
   
    while(1)
    { 
        // get reading from TMP36 via ADC
        ADCSRA |= (1 << ADSC); // start ADC measurement
        while (ADCSRA & (1 << ADSC) ); // wait till conversion complete
        _delay_ms(10);

        // get value from ADC register, convert to 8-bit value
        ADCvalue = ADC >> 2;

        // convert reading to temperature value (Celsius)
        voltage = (ADCvalue * (5000 / 256));
        temperature = (voltage - 500) / 10;	
	  
	// now you have a temperature value, take action
	if (temperature<25)
	{
            motorOff(); // under 25 degrees, turn motor off
        } 
        else if ((temperature>=25) & (temperature <35))
        {
            motorPWM(127); // between 25 and under 35 degrees, set motor to 50% PWM
        }
        else if (temperature>=35)
        {
            motorOn();     // 35 degrees and over, motor turned full on 
        }
        _delay_ms(500);	 // prevent rapid motor speed changes
    }	
}