// Project 36 - DC Motor Control with L293D

#include <avr/io.h>
#include <util/delay.h>

void initPWM(void)
{
    TCCR2A |= (1 << WGM20);  // Fast PWM mode 
    TCCR2A |= (1 << WGM21);  // Fast PWM mode, pt.2 
    TCCR2B |= (1 << CS21);   // PWM Freq = F_CPU/8/256 
}

void motorForward(uint8_t duty)
{
    // set direction
    PORTB |= (1 << PORTB1);  // PB1 HIGH
    PORTB &= ~(1 << PORTB2); // PB2 LOW

    // set speed
    if (duty == 255)
    {
        PORTB |= (1 << PORTB3);	// set PORTB3 to on
    } else
    {
        TCCR2A |= (1 << COM2A1); // PWM output on OCR2A - PB3	
        OCR2A = duty;			// set PORTB3 to PWM value
    }
}

void motorBackward(uint8_t duty)
{
    // set direction
    PORTB &= ~(1 << PORTB1);  // PB1 LOW
    PORTB |= (1 << PORTB2);   // PB2 HIGH
	
    // set speed
    if (duty == 255)
    {
        PORTB |= (1 << PORTB3);	// set PORTB3 to on
    } else
    {
        TCCR2A |= (1 << COM2A1); // PWM output on OCR2A - PB3	
        OCR2A = duty;			// set PORTB3 to PWM value
    }	
}

void motorOff(void)
{
    TCCR2A &= ~(1 << COM2A1); // disconnect PWM output from OCR2A - PB3	
    PORTB &= ~(1 << PORTB3);  // set enable to zero for brake
}

void setup()
{
}

int main(void)
{
    DDRB |= (1 << PORTB3)|(1 << PORTB2)|(1 << PORTB1); // set PORTB3, 2 and 1 as outputs
    initPWM();
    _delay_ms(3000); // wait a moment before starting
    while(1)
    {
        motorForward(64);
        _delay_ms(2000);
        motorOff();
        _delay_ms(2000);
        motorForward(255);
        _delay_ms(2000);
        motorOff();
        _delay_ms(2000);
        motorBackward(64);
        _delay_ms(2000);
        motorOff();
        _delay_ms(2000);
        motorBackward(255);
        _delay_ms(2000);
        motorOff();
        _delay_ms(2000);
    }
}