// Project 52 - Using Character LCD with AVR

#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>

void commandLCD(uint8_t _command )	
{
// takes command byte and sends upper nibble, then lowert nibble to LCD
    PORTD = (PORTD & 0x0F) | (_command & 0xF0); // sends upper nibble of command byte
    PORTD &= ~(1<<PD0);	// RS off - selects instruction register (not data register)
    PORTD |= (1<<PD1);		// E on - the E on/off precedes sending data to LCD
    _delay_us(1);			
    PORTD &= ~(1<<PD1);	// E off
    _delay_us(200);			//
    PORTD = (PORTD & 0x0F) | (_command << 4);	// sends lower nibble of command byte
    PORTD |= (1<<PD1);		// E on    
    _delay_us(1);
    PORTD &= ~(1<<PD1);	// E off
    _delay_ms(2);
}

void initLCD()
{
    DDRD = 0b11111111;  // set PORTD to outputs
    _delay_ms(100);		// wait for LCD 
	
    commandLCD(0x02);	// return cursor home
    commandLCD(0x28);   // use 4-bit control via PD7~PD4, 2 line LCD, font 0 (5x8 pixel)
    commandLCD(0x0C);   // display on, block cursor off, blinking cursor off
    commandLCD(0x06);   // tells cursor to move in incremental stages
    commandLCD(0x01);   // clear the LCD

    _delay_ms(2);		// wait for LCD
}

void clearLCD()				// check datasheet for functions
{
    commandLCD (0x01);		// clear the LCD
    _delay_ms(2);			// wait for LCD
    commandLCD (0x80);		// move cursor to top-left of LCD
    _delay_ms(2);			// wait for LCD
}

void printLCD(char *_string)	
{
    uint8_t i;
    for(i=0; _string[i]!=0; i++)
    {
        PORTD = (PORTD & 0x0F) | (_string[i] & 0xF0);
        PORTD |= (1<<PD0);
        PORTD |= (1<<PD1);
        _delay_us(1);
        PORTD &= ~(1<<PD1);
        _delay_us(200);
        PORTD = (PORTD & 0x0F) | (_string[i] << 4);
        PORTD |= (1<<PD1);
        _delay_us(1);
        PORTD &= ~(1<<PD1);
        _delay_ms(2);
    }
}

void cursorLCD(uint8_t column, uint8_t row) 	
// move cursor to column (0~15), row (0~1) 
{
    if (row == 0 && column<16)
    {
        commandLCD((column & 0x0F)|0x80); 
    }			
    else if (row == 1 && column<16)
    {
        commandLCD((column & 0x0F)|0xC0);
    }			
}

int main()
{
    initLCD();
    char numbers[9];
    uint8_t i;
	
    while(1) 
    {
        cursorLCD(1,0);
        printLCD("AVR Workshop!");
        cursorLCD(0,1);
        printLCD("Learning LCD use");
        _delay_ms(1000);
        clearLCD();
		
        cursorLCD(1,0);
        printLCD("Counting up:");
        for (i = 0; i<10; i++)
        {
            itoa(i,numbers,10);
            cursorLCD(i,1);
            printLCD(numbers);	
            _delay_ms(1000);
        }
        clearLCD();
    }
}