// Project 54 - LCD Digital Thermometer with min/max display

#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include <math.h>

void startADC()
// Set up the ADC
{
    ADMUX |= (1 << REFS0);                 // Use AVcc pin with ADC
    ADMUX |= (1 << MUX2) | (1 << MUX0);    // use ADC5 (pin 28)
    ADCSRA |= (1 << ADPS2) |(1 << ADPS1) | (1 << ADPS0); // prescaler for 16MHz (/128)	
    ADCSRA |= (1 << ADEN);                 // enable ADC
}

void commandLCD(uint8_t _command )	
{
    // takes command byte and sends upper nibble, then lowert nibble to LCD
    PORTD = (PORTD & 0x0F) | (_command & 0xF0); 
    PORTD &= ~(1<<PD0);	
    PORTD |= (1<<PD1);	
    _delay_us(1);			
    PORTD &= ~(1<<PD1);	
    _delay_us(200);		
    PORTD = (PORTD & 0x0F) | (_command << 4);	
    PORTD |= (1<<PD1);		
    _delay_us(1);
    PORTD &= ~(1<<PD1);	
    _delay_ms(2);
}

void initLCD()
{
    DDRD = 0b11111111;  
    _delay_ms(100);		
    commandLCD(0x02);	
    commandLCD(0x28);   
    commandLCD(0x0C);  
    commandLCD(0x06);  
    commandLCD(0x01); 
    _delay_ms(2);	
}

void clearLCD()			
{
    commandLCD (0x01);		
    _delay_ms(2);		
    commandLCD (0x80);	
    _delay_ms(2);		
}

void printLCD(char *_string)	
{
    uint8_t i;
    for(i=0; _string[i]!=0; i++)
    {
        PORTD = (PORTD & 0x0F) | (_string[i] & 0xF0);
        PORTD |= (1<<PD0);
        PORTD |= (1<<PD1);
        _delay_us(1);
        PORTD &= ~(1<<PD1);
        _delay_us(200);
        PORTD = (PORTD & 0x0F) | (_string[i] << 4);
        PORTD |= (1<<PD1);
        _delay_us(1);
        PORTD &= ~(1<<PD1);
        _delay_ms(2);
    }
}

void cursorLCD(uint8_t column, uint8_t row) 	
// move cursor to column (0~15), row (0~1) 
{
        if (row == 0 && column<16)
        {
            commandLCD((column & 0x0F)|0x80); 
        }			
        else if (row == 1 && column<16)
        {
            commandLCD((column & 0x0F)|0xC0);
        }			
}

int main()
{
    DDRC = 0b00000000; // set PORTC as inputs
    startADC();
    initLCD();
    char numbers[9];

    float temperature = 0;
    float voltage = 0;
    float average = 0;
    float minimum = 25; // needs an initial value
    float maximum = 0;
    uint16_t ADCvalue;

    while(1) 
    {
        // Take reading from TMP36 via ADC
        ADCSRA |= (1 << ADSC);         // Start ADC measurement
        while (ADCSRA & (1 << ADSC) ); // Wait for conversion to finish 
        _delay_ms(10);

        // Get value from ADC (which is 10-bit) register, store in ADCvalue
        ADCvalue = ADC;
	
        // convert reading to temperature value (Celsius)
        voltage = (ADCvalue * 5);
        voltage = voltage / 1024;
        temperature = ((voltage - 0.5) * 100);	
		
        // min max average
        if (temperature < minimum)
        {
            minimum = temperature;
        }
        if (temperature > maximum)
        {
            maximum = temperature;
        }
        average = ((minimum+maximum)/2);
		
        // display information
        cursorLCD(0,0);
        printLCD("Current:");
        dtostrf(temperature,6,2,numbers); 
        printLCD(numbers);
        cursorLCD(15,0);
        printLCD("C");
		
        cursorLCD(0,1);
        printLCD("Average:");
        dtostrf(average,6,2,numbers); 
        printLCD(numbers);
        cursorLCD(15,1);
        printLCD("C");
		
        _delay_ms(1000);
        clearLCD();
		
        cursorLCD(0,0);
        printLCD("Minimum:");
        dtostrf(minimum,6,2,numbers); 
        printLCD(numbers);
        cursorLCD(15,0);
        printLCD("C");
		
        cursorLCD(0,1);
        printLCD("Maximum:");
        dtostrf(maximum,6,2,numbers); 
        printLCD(numbers);
        cursorLCD(15,1);
        printLCD("C");
        _delay_ms(1000);
        clearLCD();				
    }
}